import pykube
from pykube.objects import NamespacedAPIObject


def get_api():

    config = pykube.KubeConfig.from_env()
    return pykube.HTTPClient(config)


class Broker(NamespacedAPIObject):

    version = "eventing.knative.dev/v1"
    kind = "Broker"
    endpoint = "brokers"


def get_cluster_brokers(flat=False):

    api = get_api()
    available_brokers = []
    for broker in Broker.objects(api).all():
        if flat:
            available_brokers.append(broker.name)
        else:
            available_brokers.append((broker.name, broker.name))
    return available_brokers


class Trigger(NamespacedAPIObject):

    version = "eventing.knative.dev/v1"
    kind = "Trigger"
    endpoint = "triggers"


class ContainerSource(NamespacedAPIObject):

    version = "sources.knative.dev/v1"
    kind = "ContainerSource"
    endpoint = "containersources"



import typing
from django.http import HttpRequest
from rest_framework import permissions
from django.conf import settings


class SimpleApiKeyPermissionClass(permissions.BasePermission):
    """
    This is an abstract class ment to be used in a typcal krules environment
    where the api key is provided in a configuration provider, then
    injected in django settings.
    The configuration value could be expressed as a list to facilitate the transition
    to a new value without interrupting the operation of the client services
    which receive the same configuration but potentially not at the exact same moment
    The client uses the first element of the list
    """

    settings_key: typing.Union[str, typing.List] = None

    def has_permission(self, request: HttpRequest, view: typing.Any) -> bool:

        assert self.settings_key is not None, (
            f"{self.__class__.__name__} must define a settings_key attribute"
        )

        # get key from authorization header
        authorization = request.META.get("HTTP_AUTHORIZATION")

        if not authorization:
            return False

        try:
            _, key = authorization.split("Api-Key ")
        except ValueError:
            return False

        value = getattr(settings, self.settings_key)
        if isinstance(value, typing.List):
            return key in getattr(settings, self.settings_key)
        return key == value



from django.db.models import Q
from django.contrib import admin


class DeepSearchModelAdmin(admin.ModelAdmin):
    deep_search_fields = ()

    def get_search_results(self, request, queryset, search_term):

        deep_filtered_queryset = None
        default_queryset, use_distinct = super(DeepSearchModelAdmin, self).get_search_results(request, queryset,
                                                                                              search_term)
        if len(default_queryset) == 0 and len(self.deep_search_fields) > 0:
            search_term = search_term.replace(" ", "").replace("\"", "").replace("'", "")
            items = search_term.split(",")
            for i in items:
                if deep_filtered_queryset is None:
                    qs = queryset
                else:
                    qs = deep_filtered_queryset
                try:
                    k, v = i.split(":")
                    deep_filtered_queryset = qs.filter(self._get_deep_filters(k, v))
                    if len(deep_filtered_queryset) == 0:
                        break
                except ValueError:
                    continue
        return deep_filtered_queryset or default_queryset, use_distinct

    def _get_deep_filters(self, key, value):
        filters = None
        for f in self.deep_search_fields:
            # value get from search_term is a string so we check also the evaluated value
            if filters is None:
                filters = Q(**{"%s__%s" % (f, key): value}) | Q(**{"%s__%s" % (f, key): eval(value)})
            else:
                filters = filters | Q(**{"%s__%s" % (f, key): value}) | Q(**{"%s__%s" % (f, key): eval(value)})

        return filters


from django.db.models import JSONField


class EmptyObjectJSONField(JSONField):
    empty_values = [None, ""]

    def formfield(self, **kwargs):
        from .forms import EmptyObjectJSONField

        return super().formfield(**{"form_class": EmptyObjectJSONField, **kwargs})


from django.forms import widgets
from prettyjson import PrettyJSONWidget


class ReadOnlyJSONWidget(PrettyJSONWidget):

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}
        attrs["initial"] = "parsed"
        super(ReadOnlyJSONWidget, self).__init__(attrs)

    @property
    def media(self):
        widget_media = super(ReadOnlyJSONWidget, self).media
        js = widget_media._js
        js.append('krules/prettyjson.js')
        return widgets.Media(
            js=tuple(js),
            css=widget_media._css,
        )



from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class CommonAppConfig(AppConfig):
    name = 'krules_djangoapps_common'
    verbose_name = _('KRules commons')


from setuptools import setup, find_packages
import os

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='krules-djangoapps-common',
version="0.10",    author="Airspot",
    author_email="info@arispot.tech",
    license="Apache Licence 2.0",
    keywords="krules knative",
    url="https://github.com/airspot-dev/krules",
    packages=find_packages(),
    long_description=read('README.md'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: Apache Software License",
    ],
    python_requires='>3.8',
    install_requires=[
        'django-prettyjson==0.4.1',
        'djangorestframework>=3.12.4',
        'pykube-ng==20.10.0',
    ],
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest',
    ],
)